from django.db.models import (
    QuerySet as DjangoQuerySet,
    Manager as DjangoManager,
    When,
    Value,
    Case,
)

__all__ = (
    "QuerySet",
    "Manager",
)


class QuerySet(DjangoQuerySet):
    def annotate_choices(self, output_dict=None):
        """
        choices가 존재하는 필드들의 get_FOO_display() 값을 annotate처리
        output_dict에 field.name을 사용해 별도의 output_field를 지정할 수 있다
        :param output_dict: {field.name: Field()}
        :return: QuerySet instance
        """
        annotate_dict = {}
        for field in self.model._meta.fields:
            if field.choices:
                case_args = [
                    When(**{field.name: k, "then": Value(v)})
                    for k, v in field.flatchoices
                ]
                case_kwargs = {"output_field": field.__class__()}
                if output_dict and field.name in output_dict:
                    case_kwargs["output_field"] = output_dict[field.name]
                annotate_dict[f"{field.name}_display"] = Case(
                    *case_args, **case_kwargs,
                )
        return self.annotate(**annotate_dict)


class Manager(DjangoManager.from_queryset(QuerySet)):
    pass



from drf_yasg.utils import swagger_auto_schema
from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet as DefaultGenericViewSet

__all__ = (
    "ListModelViewSet",
    "CreateModelViewSet",
    "RetrieveModelViewSet",
    "UpdateModelViewSet",
    "DestroyModelViewSet",
    "RetrieveUpdateModelViewSet",
    "RetrieveUpdateDestroyModelViewSet",
    "ReadOnlyModelViewSet",
    "ModelViewSet",
)


class GetSerializerClassMixin:
    def get_serializer_class(self):
        try:
            action = getattr(self, "action")
            method = getattr(getattr(self, "request"), "method")

            if action:
                if action == "partial_update":
                    action = "update"
                return self.serializer_classes[action]
            if method:
                return self.serializer_classes[self.request.method]
        except (KeyError, AttributeError):
            pass
        return super().get_serializer_class()


class ViewSetMixin(GetSerializerClassMixin):
    @swagger_auto_schema(auto_schema=None)
    def update(self, request, *args, **kwargs):
        return super().update(request, *args, **kwargs)


class ListModelViewSet(ViewSetMixin, mixins.ListModelMixin, DefaultGenericViewSet):
    pass


class CreateModelViewSet(ViewSetMixin, mixins.CreateModelMixin, DefaultGenericViewSet):
    pass


class RetrieveModelViewSet(
    ViewSetMixin, mixins.RetrieveModelMixin, DefaultGenericViewSet
):
    pass


class UpdateModelViewSet(ViewSetMixin, mixins.UpdateModelMixin, DefaultGenericViewSet):
    pass


class DestroyModelViewSet(
    ViewSetMixin, mixins.DestroyModelMixin, DefaultGenericViewSet
):
    pass


class RetrieveUpdateModelViewSet(
    ViewSetMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    DefaultGenericViewSet,
):
    pass


class RetrieveUpdateDestroyModelViewSet(
    ViewSetMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    DefaultGenericViewSet,
):
    pass


class ReadOnlyModelViewSet(
    ViewSetMixin,
    mixins.RetrieveModelMixin,
    mixins.ListModelMixin,
    DefaultGenericViewSet,
):
    pass


class ModelViewSet(
    ViewSetMixin,
    mixins.CreateModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    mixins.ListModelMixin,
    DefaultGenericViewSet,
):
    pass



from typing import Sequence, Tuple

from django.utils.decorators import method_decorator
from drf_yasg.utils import swagger_auto_schema

__all__ = ("schema",)


def schema(view, methods: Sequence[Tuple[str, dict]]):
    """
    drf-yasg's documentation helper function
    :param view:
    :param methods:
    :return:
    """
    for method in methods:
        name = method[0]
        if name == "update":
            name = "partial_update"
        view = method_decorator(name=name, decorator=swagger_auto_schema(**method[1]))(
            view
        )
    return view



from django.middleware.common import CommonMiddleware

__all__ = ("AppendSlashMiddleware",)


class AppendSlashMiddleware(CommonMiddleware):
    """
    If there is no '/' at the end of the request URL,
    it is forcibly attached.
    """

    def process_request(self, request):
        if not request.path.endswith("/"):
            request.path += "/"
        if not request.path_info.endswith("/"):
            request.path_info += "/"



from typing import Type

from django.db import models
from rest_framework import serializers

__all__ = (
    "PkModelFieldDoesNotExist",
    "PkModelFieldIsNotForRead",
    "PkModelField",
)


class PkModelFieldDoesNotExist(Exception):
    def __init__(self, model, pk):
        self.model = model
        self.msg = '"{model_class}"에서 pk={pk}인 인스턴스를 찾을 수 없습니다'.format(
            model_class=model.__class__.__name__, pk=pk
        )

    def __str__(self):
        return self.msg


class PkModelFieldIsNotForRead(Exception):
    def __str__(self):
        return "PkModelField는 생성시에만 사용할 수 있습니다"


class PkModelField(serializers.Field):
    """
    특정 Model의 instance를 나타내는 Field
    주어지는 데이터가
        dict
            {'pk': 1} 또는 {'id': 1}
        literal (pk가 될 수 있는 숫자/문자 등의 고정 값)
            1, '1'
    과 같은 여러 형태일 때 전부 처리해주도록 한다

    * 생성시에만 사용한다
    """

    def __init__(self, model: Type[models.Model], *args, **kwargs):
        """
        :param model: Model class
        :param kwargs:
        """
        if not issubclass(model, models.Model):
            raise ValueError('PkModelField의 "model"인수에는 Django Model의 자식클래스가 전달되어야합니다')

        self.model = model
        super().__init__(*args, **kwargs)

    def to_internal_value(self, data):
        if not isinstance(data, self.model):
            if hasattr(data, "get"):
                pk = (
                    data.get("id")
                    or data.get("pk")
                    or data.get(self.model._meta.pk.attname, data)
                )
            else:
                pk = data

            try:
                return self.model.objects.get(pk=pk)
            except self.model.DoesNotExist:
                raise PkModelFieldDoesNotExist(self.model, pk)
        return data

    def to_representation(self, value):
        raise PkModelFieldIsNotForRead()



#!/usr/bin/env python
import os
from setuptools import find_packages, setup

ROOT_DIR = os.path.dirname(os.path.dirname(__file__))
README = open(os.path.join(ROOT_DIR, "README.md")).read()
VERSION = open(os.path.join(ROOT_DIR, "version.txt")).read()

setup(
    name="django-aid",
    version=VERSION,
    description="Helper module used in general django applications",
    long_description=README,
    long_description_content_type="text/markdown",
    author="lhy",
    author_email="dev@lhy.kr",
    license="MIT",
    packages=find_packages(exclude=["test*", "sample", "demo"]),
    install_requires=["django",],
    python_requires=">=3.7",
    url="https://github.com/LeeHanYeong/django-aid",
    zip_safe=True,
    classifiers=[
        "Framework :: Django",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



from rest_framework.renderers import BrowsableAPIRenderer

__all__ = ("BrowsableAPIRendererWithoutForms",)


class BrowsableAPIRendererWithoutForms(BrowsableAPIRenderer):
    def get_context(self, *args, **kwargs):
        context = super(BrowsableAPIRendererWithoutForms, self).get_context(
            *args, **kwargs
        )
        context["display_edit_forms"] = False
        return context

    def get_filter_form(self, data, view, request):
        return None

    def get_rendered_html_form(self, data, view, method, request):
        return None
